#ifndef _ttail_search_h__
#define _ttail_search_h__

#include <errno.h>
#include <string.h>
#include <time.h>
#include <regex.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

typedef union _ttail_search_u ttail_search_t;

#include "config.h"
#include "ttail.h"

typedef char* (*ttail_search_subst_f)(ttail_t*, const char*);

typedef struct _ttail_search_file_s ttail_search_file_t;
typedef struct _ttail_search_stdin_s ttail_search_stdin_t;

/*<! Private search session for logfiles */
struct _ttail_search_file_s
{
	/*<! logfile sizes */
	off_t *file_sz;
	#ifdef TTAIL_HUGE_FILE
	/*<! Shift width to apply on size to compute stuff */
	short sz_div;
	#endif
	/*<! Computed files start size */
	off_t *vfile;
	/*<! Computed file sizes sum */
	off_t vsz;
	/*<! Computed position */
	off_t vpos;
};

/*<! Private search session for stdin */
struct _ttail_search_stdin_s
{
	char *buff;
	size_t buff_sz;
};

/**@brief Search session is an union depending in the search method
 * employed
 */
union _ttail_search_u
{
	ttail_search_file_t file;
	ttail_search_stdin_t std;
};

/**@brief Comparison results
 *
 * -1 if a < b, 1 if a > b, 0 if a == b, -2 if exception
 *
 * If an exception occur, exception flags are set with | on the result.
 * Exceptions flags are : TTAIL_NODATE_A and TTAIL_NODATE_B
 */
typedef int ttail_cmp_res;

/**@brief Begin a search session placing it in a ready to print situation
 *@param ttail_t*
 *@return 0 if ok -1 if fatal error 1 if not found
 */
int ttail_search_closest(ttail_t*);

/**@brief @ref ttail_search_closest() implementation for logfiles
 *@param ttail_t*
 *@return 0 if ok -1 if fatal error 1 if not found
 */
int _ttail_search_closest_files(ttail_t*);
int _ttail_search_closest_files_init(ttail_t*);
int _ttail_search_closest_files_set_fsizes(ttail_t*);

/**@brief @ref ttail_search_closest() implementation for stdin
 *@param ttail_t*
 *@return 0 if ok -1 if fatal error 1 if not found
 */
int _ttail_search_closest_stdin(ttail_t*);

/**@brief Attempt to reopen a file
 *@param ttail_t* ttail
 *@param size_t id file id in ttail
 *@return 0 on success, -1 on failure and errno is set
 *@throw EINVAL if id is too big
 */
int _ttail_file_reopen(ttail_t*, size_t);

/**@brief Loglines comparison function
 *@param ttail_t* ttail
 *@param const char* logline a
 *@param const char* logline b
 *@return @ref ttail_cmp_res
 */
ttail_cmp_res ttail_cmp_loglines(ttail_t*, const char*, const char*);

/**@brief compare a logline to a date
 *@param ttail_t* ttail
 *@param const char* logline
 *@param struct tm* @see time.h
 *@return @ref ttail_cmp_res
 */
ttail_cmp_res ttail_cmp_logline(ttail_t*, const char*, struct tm*);

/**@brief Fetch a date from a logline
 *@param ttail_t* ttail
 *@param const char* logline
 *@param struct tm* tm will be set to extracted date if not NULL
 *@return 0 if ok, -1 if error
 */
int ttail_logline2date(ttail_t*, const char*, struct tm*);

/**@brief Apply the choosen substitution to the logline
 *@param ttail_t* ttail
 *@param const char* logline
 *@return a pointer on the end of the subst string
 */
const char* ttail_logline_subst(ttail_t*, const char*);

/**@brief Regex logline prefix substitution
 *@param ttail_t* ttail
 *@param const char* logline
 *@return a pointer on the end of the subst string
 */
const char* _ttail_logline_subst_re(ttail_t*, const char*);

/**@brief Static len logline prefix substitution
 *@param ttail_t* ttail
 *@param const char* logline
 *@return a pointer on the end of the subst string
 */
const char* _ttail_logline_subst_len(ttail_t*, const char*);

#endif
